package org.apache.commons.lang3;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Serializable;
import org.apache.commons.lang3.SerializationException;
import org.apache.commons.lang3.Validate;

public class SerializationUtils {
  public static <T extends Serializable> T clone(T object) {
    if (object == null)
      return null; 
    byte[] objectData = serialize((Serializable)object);
    ByteArrayInputStream bais = new ByteArrayInputStream(objectData);
    try (ClassLoaderAwareObjectInputStream in = new ClassLoaderAwareObjectInputStream(bais, object
          .getClass().getClassLoader())) {
      Serializable serializable = (Serializable)in.readObject();
      return (T)serializable;
    } catch (ClassNotFoundException ex) {
      throw new SerializationException("ClassNotFoundException while reading cloned object data", ex);
    } catch (IOException ex) {
      throw new SerializationException("IOException while reading or closing cloned object data", ex);
    } 
  }
  
  public static <T> T deserialize(byte[] objectData) {
    Validate.notNull(objectData, "objectData", new Object[0]);
    return deserialize(new ByteArrayInputStream(objectData));
  }
  
  public static <T> T deserialize(InputStream inputStream) {
    Validate.notNull(inputStream, "inputStream", new Object[0]);
    try (ObjectInputStream in = new ObjectInputStream(inputStream)) {
      T obj = (T)in.readObject();
      return obj;
    } catch (ClassNotFoundException|IOException ex) {
      throw new SerializationException(ex);
    } 
  }
  
  public static <T extends Serializable> T roundtrip(T obj) {
    return (T)deserialize(serialize((Serializable)obj));
  }
  
  public static byte[] serialize(Serializable obj) {
    ByteArrayOutputStream baos = new ByteArrayOutputStream(512);
    serialize(obj, baos);
    return baos.toByteArray();
  }
  
  public static void serialize(Serializable obj, OutputStream outputStream) {
    Validate.notNull(outputStream, "outputStream", new Object[0]);
    try (ObjectOutputStream out = new ObjectOutputStream(outputStream)) {
      out.writeObject(obj);
    } catch (IOException ex) {
      throw new SerializationException(ex);
    } 
  }
}
